/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005/2006, Anthony Minessale II <anthmct@yahoo.com>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthmct@yahoo.com>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Anthony Minessale II <anthmct@yahoo.com>
 * Michael Jerris <mike@jerris.com>
 * Ken Rice <krice at suspicious dot org>
 * Matt Klein <mklein@nmedia.net>
 *
 * mod_slimit.c -- Resource Limit Module - Slim-line revision
 *
 */

#include <switch.h>

SWITCH_MODULE_LOAD_FUNCTION(mod_slimit_load);
SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_slimit_shutdown);
SWITCH_MODULE_DEFINITION(mod_slimit, mod_slimit_load, mod_slimit_shutdown, NULL);

struct slimit_node {
    char *id;
    int count;
};

typedef struct slimit_node slimit_node_t;


static struct {
	switch_memory_pool_t *pool;
	switch_mutex_t *mutex;
	switch_hash_t *hash;

	int running;

	slimit_node_t *node;

} globals;




static slimit_node_t *create_node(const char *id)
{   
    slimit_node_t *node;  

    if (!globals.running) {
        return NULL;
    }
	
	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "create_node %s!", id);

    node = switch_core_alloc(globals.pool, sizeof(*node));

    switch_core_hash_insert(globals.hash, id, node);
    node->id = switch_core_strdup(globals.pool, id);
    node->count = 1;

    return node;
} 




static switch_status_t hanguphook(switch_core_session_t *session)
{
	switch_channel_t *channel = switch_core_session_get_channel(session);
	switch_channel_state_t state = switch_channel_get_state(channel);
	slimit_node_t *node = NULL;
	const char *id = NULL;
	const char *max = NULL;

	if (state == CS_HANGUP || state == CS_ROUTING) {
		id = switch_channel_get_variable(channel, "slimit_id");
		max = switch_channel_get_variable(channel, "slimit_max");

		/* Remove one count from the struct */
    	if (!(node = switch_core_hash_find(globals.hash, id))) {
    	    node = create_node(id);
    	}

		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "hanguphook %s %d %s!", id, node->count, max);

		node->count--;

		switch_core_event_hook_remove_state_change(session, hanguphook);
	}
	return SWITCH_STATUS_SUCCESS;
}


#define SLIMIT_USAGE "<id> <max> [max_transfer_extension (slimit_exceeded is default)]"
#define SLIMIT_DESC "Limit concurrent calls to an ID within Dialplan"
static char *slimit_def_xfer_exten = "slimit_exceeded";

SWITCH_STANDARD_APP(slimit_function)
{
	int argc = 0;
	char *argv[6] = { 0 };
	char *mydata = NULL;
	char *id = NULL;
	char *xfer_exten = NULL;
	int max = 0, got = 0;
	slimit_node_t *node = NULL;

	switch_channel_t *channel = switch_core_session_get_channel(session);

	if (!switch_strlen_zero(data)) {
		mydata = switch_core_session_strdup(session, data);
		argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0])));
	}

	if (argc < 2) {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "USAGE: slimit %s\n", SLIMIT_USAGE);
		return;
	}

	switch_mutex_lock(globals.mutex);

	id = argv[0];
	max = atoi(argv[1]);

	if (argc == 3) {
		xfer_exten = argv[2];
	} else {
		xfer_exten = slimit_def_xfer_exten;
	}

	if (max < 0) {
		max = 0;
	}

    if (!(node = switch_core_hash_find(globals.hash, id))) {
        node = create_node(id);
    } else {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "Found a non-node %s!!!!\n", id);	
	}

	/* get count for id */
	got = node->count;
	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "Found a count value %d for max %d!!!!\n", node->count, max);	

	switch_channel_set_variable(channel, "slimit_id", id);
	switch_channel_set_variable(channel, "slimit_max", argv[1]);
	switch_channel_set_variable(channel, "slimit_current", switch_mprintf("%d", got));

	if (got > max) {
		switch_ivr_session_transfer(session, xfer_exten, NULL, NULL);
		goto done;
	}

	/* up the count */
	got += 1;

	switch_channel_set_variable(channel, "slimit_current", switch_mprintf("%d", got));

	/* Set the hook to remove a count for an id */
	switch_core_event_hook_add_state_change(session, hanguphook);

	/* note: got == original got + 1 */
	node->count = got;

  done:
	switch_mutex_unlock(globals.mutex);
}


SWITCH_MODULE_LOAD_FUNCTION(mod_slimit_load)
{
	/* left over from config
		switch_status_t status;
	*/

	switch_application_interface_t *app_interface;

	/* old
	memset(&globals, 0, sizeof(&globals));
	gethostname(globals.hostname, sizeof(globals.hostname));
	globals.pool = pool;
	*/
	/* don't need config here .... all in the DP
		if ((status = do_config() != SWITCH_STATUS_SUCCESS)) {
			return status;
		}
	*/
	switch_core_new_memory_pool(&globals.pool);
	switch_core_hash_init(&globals.hash, globals.pool);
	switch_mutex_init(&globals.mutex, SWITCH_MUTEX_NESTED, globals.pool);

	/* connect my internal structure to the blank pointer passed to me */
	*module_interface = switch_loadable_module_create_module_interface(pool, modname);

	SWITCH_ADD_APP(app_interface, "slimit", "slimit", SLIMIT_DESC, slimit_function, SLIMIT_USAGE, SAF_SUPPORT_NOMEDIA);

	globals.running = 1;

	/* indicate that the module should continue to be loaded */
	return SWITCH_STATUS_SUCCESS;
}


SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_slimit_shutdown)
{
    switch_memory_pool_t *pool = globals.pool;
    switch_mutex_t *mutex = globals.mutex;

    switch_mutex_lock(mutex);

    globals.running = 0;

    /* Cleanup */
    switch_core_hash_destroy(&globals.hash);
    memset(&globals, 0, sizeof(globals));
    switch_mutex_unlock(mutex);
    switch_core_destroy_memory_pool(&pool);
    return SWITCH_STATUS_SUCCESS;
}



/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
 */
